import React, { useEffect, useState } from 'react'
import { Link, useParams } from 'react-router-dom'

import Page from 'components/page'
import { useTranslation } from 'react-i18next'
import {
  Button,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableHead,
  TableRow,
  Paper,
} from '@mui/material'
import { Guest, Role, FetchedRole } from 'interfaces'
import SponsorInfoButtons from 'routes/components/sponsorInfoButtons'
import { format } from 'date-fns'
import { parseRole } from 'utils'

type GuestInfoParams = {
  pid: string
}
interface RoleLineProps {
  role: Role
  pid: string
}

const RoleLine = ({ role, pid }: RoleLineProps) => {
  const [t, i18n] = useTranslation('common')
  return (
    <TableRow
      key={role.id}
      sx={{ '&:last-child td, &:last-child th': { border: 0 } }}
    >
      <TableCell align="left">
        {i18n.language === 'en' ? role.name_en : role.name_nb}
      </TableCell>
      <TableCell component="th" scope="row">
        {role.start_date ? format(role.start_date, 'yyyy-MM-dd') : null} -{' '}
        {format(role.end_date, 'yyyy-MM-dd')}
      </TableCell>
      <TableCell align="left">
        {i18n.language === 'en' ? role.ou_en : role.ou_nb}
      </TableCell>
      <TableCell>
        <Button
          variant="contained"
          component={Link}
          to={`/sponsor/guest/${pid}/roles/${role.id}`}
        >
          {t('sponsor.choose')}
        </Button>
      </TableCell>
    </TableRow>
  )
}

export default function GuestInfo() {
  const { pid } = useParams<GuestInfoParams>()
  const [t] = useTranslation(['common'])
  const [guestInfo, setGuest] = useState<Guest>({
    pid: '',
    first: '',
    last: '',
    email: '',
    fnr: '',
    mobile: '',
    active: false,
    registered: false,
    verified: false,
    roles: [],
  })
  const [roles, setRoles] = useState<Role[]>([])

  const getPerson = async (id: string) => {
    try {
      const response = await fetch(`/api/ui/v1/person/${id}`)
      const rjson = await response.json()
      if (response.ok) {
        setGuest({
          pid: rjson.pid,
          first: rjson.first,
          last: rjson.last,
          email: rjson.email,
          mobile: rjson.mobile,
          fnr: rjson.fnr,
          active: rjson.active,
          registered: rjson.registered,
          verified: rjson.verified,
          roles: rjson.roles,
        })
        setRoles(rjson.roles.map((role: FetchedRole) => parseRole(role)))
      }
    } catch (error) {
      console.error(error)
    }
  }

  useEffect(() => {
    getPerson(pid)
  }, [])

  return (
    <Page>
      <SponsorInfoButtons to="/sponsor" />
      <h4>{t('sponsor.contactInfo')}</h4>
      <TableContainer component={Paper}>
        <Table sx={{ minWidth: 650 }} aria-label="simple table">
          <TableHead sx={{ backgroundColor: 'primary.light' }}>
            <TableRow>
              <TableCell align="left">{t('sponsor.contactInfo')}</TableCell>
              <TableCell />
            </TableRow>
          </TableHead>
          <TableBody>
            <TableRow>
              <TableCell align="left">{t('input.fullName')}</TableCell>
              <TableCell align="left">
                {`${guestInfo.first} ${guestInfo.last}`}
              </TableCell>
            </TableRow>
            <TableRow>
              <TableCell align="left">{t('input.email')}</TableCell>
              <TableCell align="left">{guestInfo.email}</TableCell>
            </TableRow>
            <TableRow>
              <TableCell align="left">{t('input.nationalIdNumber')}</TableCell>
              <TableCell align="left">{guestInfo.fnr}</TableCell>
            </TableRow>
            <TableRow>
              <TableCell align="left">{t('input.mobilePhone')}</TableCell>
              <TableCell align="left">{guestInfo.mobile}</TableCell>
            </TableRow>
          </TableBody>
        </Table>
      </TableContainer>
      <h4>{t('sponsor.roleInfo')}</h4>
      <TableContainer component={Paper}>
        <Table sx={{ minWidth: 650 }} aria-label="simple table">
          <TableHead sx={{ backgroundColor: 'primary.light' }}>
            <TableRow>
              <TableCell align="left">{t('common:role')}</TableCell>
              <TableCell align="left">{t('common:period')}</TableCell>
              <TableCell align="left">{t('common:ou')}</TableCell>
              <TableCell align="left">{t('common:choice')}</TableCell>
            </TableRow>
          </TableHead>
          <TableBody>
            {roles.map((role) => (
              <RoleLine pid={pid} role={role} />
            ))}
          </TableBody>
        </Table>
      </TableContainer>
    </Page>
  )
}
