import format from 'date-fns/format'
import { addDays } from 'date-fns/fp'
import {
  Button,
  Paper,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableHead,
  TableRow,
  TextField,
} from '@mui/material'
import Page from 'components/page'
import { Guest } from 'interfaces'
import { useTranslation } from 'react-i18next'
import { useParams } from 'react-router-dom'
import SponsorInfoButtons from 'routes/components/sponsorInfoButtons'
import { DatePicker } from '@mui/lab'
import { Controller, SubmitHandler, useForm } from 'react-hook-form'
import { submitJsonOpts } from '../../../utils'

interface GuestRoleInfoProps {
  guests: Guest[]
}
const endPeriodPost = (id: string, data: { end_date: Date }) => {
  const payload = {
    end_date: format(data.end_date as Date, 'yyyy-MM-dd'),
  }
  console.log('submitting', JSON.stringify(payload))

  fetch(`/api/ui/v1/role/${id}`, submitJsonOpts('PATCH', payload))
    .then((res) => {
      if (!res.ok) {
        return null
      }
      return res.text()
    })
    .then((result) => {
      if (result !== null) {
        console.log('result', result)
      }
    })
    .catch((error) => {
      console.log('error', error)
    })
}

type GuestRoleInfoParams = {
  pid: string
  id: string
}

type RoleFormData = {
  start_date: Date
  end_date: Date
}

export default function GuestRoleInfo({ guests }: GuestRoleInfoProps) {
  const { pid, id } = useParams<GuestRoleInfoParams>()
  const [t, i18n] = useTranslation('common')

  // Find the role info relevant for this page
  const guestInfo = guests.filter((guest) => guest.pid.toString() === pid)[0]
  const roleInfo = guestInfo.roles.filter(
    (role) => role.id.toString() === id
  )[0]

  // Prepare min and max date values
  const today = new Date()
  const todayPlusMaxDays = addDays(roleInfo.max_days)(today)

  // Make a function for use with onClick of the end role button
  const endPeriod = () => () => {
    roleInfo.end_date = today
    endPeriodPost(id, { end_date: today })
  }

  // Submit function for the save button
  const submit: SubmitHandler<RoleFormData> = (data) => {
    const payload: { start_date?: string; end_date: string } = {
      end_date: format(data.end_date as Date, 'yyyy-MM-dd'),
    }
    // Ignore start dates before today ()
    if (data.start_date >= today) {
      payload.start_date = format(data.start_date as Date, 'yyyy-MM-dd')
    }

    console.log('submitting', JSON.stringify(payload))
    fetch(`/api/ui/v1/role/${id}`, submitJsonOpts('PATCH', payload))
      .then((res) => {
        if (!res.ok) {
          return null
        }
        return res.text()
      })
      .then((result) => {
        if (result !== null) {
          console.log('result', result)
        }
      })
      .catch((error) => {
        console.log('error', error)
      })
  }

  const { control, handleSubmit } = useForm()
  const onSubmit = handleSubmit(submit)

  return (
    <Page>
      <SponsorInfoButtons to={`/sponsor/guest/${pid}`} />
      <h4>{t('sponsor.roleInfoText')}</h4>
      <form onSubmit={onSubmit}>
        <TableContainer component={Paper}>
          <Table sx={{ minWidth: 650 }} aria-label="simple table">
            <TableHead sx={{ backgroundColor: 'primary.light' }}>
              <TableRow>
                <TableCell align="left">{t('sponsor.details')}</TableCell>
                <TableCell />
                <TableCell />
                <TableCell />
              </TableRow>
            </TableHead>
            <TableBody>
              <TableRow>
                <TableCell align="left">{t('common:role')}</TableCell>
                <TableCell>
                  {i18n.language === 'en' ? roleInfo.name_en : roleInfo.name_nb}
                </TableCell>
              </TableRow>
              <TableRow>
                <TableCell align="left">{t('common:period')}</TableCell>
                <TableCell align="left">
                  <Controller
                    name="start_date"
                    control={control}
                    defaultValue={roleInfo.start_date}
                    render={({ field: { onChange, value } }) => (
                      <DatePicker
                        mask="____-__-__"
                        disabled={
                          roleInfo.start_date.getDate() <= today.getDate()
                        }
                        label={t('input.roleStartDate')}
                        value={value}
                        minDate={today}
                        maxDate={todayPlusMaxDays}
                        inputFormat="yyyy-MM-dd"
                        onChange={onChange}
                        renderInput={(params) => <TextField {...params} />}
                      />
                    )}
                  />
                </TableCell>
                <TableCell align="left">
                  <Controller
                    name="end_date"
                    control={control}
                    defaultValue={roleInfo.end_date}
                    render={({ field: { onChange, value } }) => (
                      <DatePicker
                        mask="____-__-__"
                        label={t('input.roleEndDate')}
                        disabled={roleInfo.end_date.getDate() < today.getDate()}
                        minDate={today}
                        maxDate={todayPlusMaxDays}
                        value={value}
                        inputFormat="yyyy-MM-dd"
                        onChange={onChange}
                        renderInput={(params) => <TextField {...params} />}
                      />
                    )}
                  />
                </TableCell>
                <TableCell>
                  <Button onClick={endPeriod()}>{t('sponsor.endNow')}</Button>
                </TableCell>
              </TableRow>
              <TableRow>
                <TableCell align="left">{t('common:ou')}</TableCell>
                <TableCell align="left">
                  {i18n.language === 'en' ? roleInfo.ou_en : roleInfo.ou_nb}
                </TableCell>
                <TableCell align="left" />
              </TableRow>
            </TableBody>
          </Table>
        </TableContainer>
        <Button variant="contained" type="submit">
          {t('button.save')}
        </Button>
      </form>
    </Page>
  )
}
